#include <mockturtle-rt.h>

#define TEST_MESSAGE_ID 0xdeadbeef

static void send_pkt(int rmq, unsigned val)
{
	struct trtl_fw_msg msg;
	int err;

	// queue full? wait
	while (mq_claim(TRTL_RMQ, rmq) < 0)
                ;

	mq_map_out_message(TRTL_RMQ, rmq, &msg);

	unsigned *p = msg.payload;
	msg.header->len = 2;

	p[0] = TEST_MESSAGE_ID;
        p[1] = val;

	mq_send(TRTL_RMQ, rmq);
}

static unsigned handle_rx(int rmq)
{
        unsigned res;

	while (!mq_poll_in(TRTL_RMQ, 1 << rmq))
                ;

	struct trtl_fw_msg tmsg;

	mq_map_in_message(TRTL_RMQ, rmq, &tmsg);

	unsigned *p = tmsg.payload;
        res = p[1];

        pp_printf("Recv id=%x, val=%x\n", p[0], p[1]);

	mq_discard(TRTL_RMQ, rmq);

        return res;
}


int main()
{
        unsigned v;
	int rmq = 0;
	struct trtl_ep_eth_address bind_addr;

	pp_printf("RMQ UDP EP test\n");

	// set up the RMQ Endpoint

	// we operate only with UDP frames
	bind_addr.type = TRTL_EP_FRAME_UDP;

	// destination MAC: we use broadcast
	bind_addr.dst_mac[0] = 0xff;
	bind_addr.dst_mac[1] = 0xff;
	bind_addr.dst_mac[2] = 0xff;
	bind_addr.dst_mac[3] = 0xff;
	bind_addr.dst_mac[4] = 0xff;
	bind_addr.dst_mac[5] = 0xff;

	// destination port
	bind_addr.dst_port = 12345;
	// source port
	bind_addr.src_port = 7777;
	// destination IP: 192.168.90.255 (broadcast)
	bind_addr.dst_ip = 0xC0A85AFF;
	// source IP: 192.168.90.255 (broadcast)
	bind_addr.src_ip = 0xC0A85A11;
	bind_addr.ethertype = 0x800; // IPv4
	// RX filter: we want only UDP packets with matching desination port & IP address.
	bind_addr.filter = TRTL_EP_FILTER_UDP | TRTL_EP_FILTER_DST_PORT | TRTL_EP_FILTER_DST_IP;
        bind_addr.filter |= TRTL_EP_FILTER_ENABLE;

	// configure outgoing channel
	rmq_bind_out(rmq, TRTL_EP_ETH, &bind_addr);
	// configure incoming channel
	rmq_bind_in(rmq, TRTL_EP_ETH, &bind_addr);

        send_pkt(0, 0x123);
        v = handle_rx(0);
        pp_printf("rx(1): %x\n", v);
        send_pkt(0, v);
        v = handle_rx(0);
        pp_printf("rx(2): %x\n", v);

        if (v == 0x123) {
                lr_writel(v, MT_CPU_LR_REG_NTF_INT);
        }

	return 0;
}
